// WMI.cpp : Defines the entry point for the console application.
//
#include "stdafx.h"
#include <windows.h>
#include <fstream>
#include <iostream>
#include "WmiBufferCal.h"
using namespace std;

/*
Parse Method "WMBC", find the relationship about method-id <=> method name, eg. 0x53555061 <=> SLBO

Tag: "WMBC", "_T_0" (offset 0x5 => method-id, 0xA => method name), 0xA5( ACPI Table, BreakOp, the sig of method end.)

			Method(WMBC, 3, Serialized)
			{
				Name(_T_0, Zero)
				Store(One, Local0)
				While(One)
				{
					Store(Arg1, _T_0)
					If(LEqual(_T_0, 0x43455053))
					{
						Return(SPEC(Arg2))
					}
					Else
					...
					{
						If(LEqual(_T_0, 0x53555061))
						{
							Return(SLBO(Arg2))
						}
						Else
						...
					}
					Break
				}
				Return(Local0)
			}

*/
#define MethodIDOffset   0x05 // "_T_0." => 5 chars
#define MethodNameOffset 0x0A // "_T_0." => 5 chars + MethodID => 4 chars + 0xA4(ReturnOP) => 1 char = 10 chars

/*
Parse "RAMA" block, find the relationship about method name <=> WFUN ID, eg. SLBO <=> 0x80000009
			Name(RAMA, 0xC43F4D98)
			Name(WBUF, Buffer(0x50){})
			OperationRegion(\RAMX, SystemMemory, RAMA, 0x0100)

            ...

			Method(SLBO, 1, Serialized)
			{
				If(LEqual(Acquire(WMTX, 0xFFFF), Zero))
				{
					Store(0x80000009, WFUN)
					Store(Zero, WSTS)
					Store(Zero, ODAT)
					Store(Arg0, IDAT)
					\_SB.ISMI(0xDA)
					Release(WMTX)
					Store(ODAT, WBUF)
					Return(WBUF)
				}
				Return(Ones)
			}
*/

//The final data
#define MaxMethodCount 0xF
  DWORD    MethodID[MaxMethodCount], FunNum[MaxMethodCount];
  CHAR     MethodName[MaxMethodCount][5]; 
  int      MethodCount = 0;

UINT32 GetWMBC(UINT32 DSDTTableAddr)
{
//Temp Var to record the result of "WMBC" block
  DWORD    WMBCOffset = 0, WMBCAddr = 0 , WMBCBreakOffset = 0, WMBCBreakAddr = 0,  WMBCTempOffset = 0, WMBCTempAddr = 0;
  CHAR     WMBCBreakOP[2] = {0xA5,'\0'};

//Temp Var to record the result of "RAMA" block
  DWORD    WmiTagOffset = 0, WmiTagAddr = 0, MethodNameOffsetinWMI = 0, FunOffset = 0; 

//Other temp var.
  int      i = 0;

  WMBCOffset = MapMem_wcsrch( DSDTTableAddr, 0x200000, "WMBC" );
  WMBCAddr   = WMBCOffset + DSDTTableAddr;
  WMBCTempAddr = WMBCAddr;
  //printf("GetWMBC = %08x\n", WMBCAddr);
  if(WMBCOffset == -1) return 0;

  WMBCBreakOffset = MapMem_wcsrch( WMBCAddr, 0x200000, WMBCBreakOP );
  WMBCBreakAddr   = WMBCBreakOffset + WMBCAddr;
  //printf("GetWMBC End = %08x\n",WMBCBreakAddr);
  if(WMBCBreakOffset == -1) return 0;

  WmiTagOffset = MapMem_wcsrch( DSDTTableAddr, 0x200000, "RAMA" );
  WmiTagAddr   = WmiTagOffset + DSDTTableAddr;
  if(WmiTagOffset == -1) return 0;

  while(WMBCTempAddr < WMBCBreakAddr)
  {
	WMBCTempOffset = MapMem_wcsrch( WMBCTempAddr, 0x200000, "_T_0" );
	if(WMBCTempOffset == -1) return 0;

	Read_Mem_Dword(WMBCTempOffset + WMBCTempAddr + MethodIDOffset, &(MethodID[MethodCount]));
	//printf("[%08x] = %08x\n", WMBCTempAddr,MethodID[MethodCount]);

	//According Method Name (4 chars), search WFUN 
	for(i=0; i<4; i++)
	{
	  Read_Mem_Byte(WMBCTempOffset + WMBCTempAddr + MethodNameOffset + i,(PBYTE)(MethodName[MethodCount]+i));
	  //printf("%c\n", *(MethodName+i));
	}
	MethodName[MethodCount][i] = '\0';
			   
	MethodNameOffsetinWMI = MapMem_wcsrch( WmiTagAddr, 0x200000, MethodName[MethodCount]);
	if (MethodNameOffsetinWMI != -1) //if "RAMA" block exist the same MethodName, will report to output file.
	{
	  //printf("%s = %08x ", MethodName[MethodCount], WmiTagAddr + MethodNameOffsetinWMI);
	  FunOffset = MapMem_wcsrch(  WmiTagAddr + MethodNameOffsetinWMI, 0x20, "WFUN");

	  if(FunOffset != -1)
	  {
	    Read_Mem_Dword(WmiTagAddr + MethodNameOffsetinWMI + FunOffset - 4, &(FunNum[MethodCount]));//the front of "WFUN" is FunNum(Len = 4 byte)
	    //printf("%08x(%d)\n",FunNum[MethodCount],MethodCount);
		MethodCount++;
	  }
	}

	//Find the next MethodID, start address need to skip current MethodID.
	WMBCTempAddr = WMBCTempOffset + WMBCTempAddr + MethodIDOffset;
  }

  return 1;
}

/*
Name(WQMO, Buffer(0x0D40)
	{
	...
	}
*/
UINT32 GetWMIMOF(UINT32 DSDTTableAddr)
{
  DWORD    MOFOffset = 0, MOFAddr = 0;
  WORD     MOFSize = 0;

  MOFOffset = MapMem_wcsrch( DSDTTableAddr, 0x200000, "WQMO" );
  if(MOFOffset != -1)
  {
    MOFAddr   = MOFOffset + DSDTTableAddr;
    Read_Mem_Word(MOFAddr + 0x8, &MOFSize);
	//printf("%08x (%04x)\n",MOFAddr,MOFSize);

    ofstream Output;
	BYTE     Temp, Temp2;
	DWORD    temp1;
	Output.open("WmiMOF.bin",ios::binary);
    if(Output.good())
    {
      for(int i = MOFAddr + 0xA; i<MOFSize + MOFAddr + 0xA; i++)
      {
		Read_Mem_Byte(i, &Temp);
		Read_Mem_Byte(i+1, &Temp2);
		if(Temp2 == 0x0A)
		{
		  //printf("%x ",i);
		  
		  //printf("%02x \n",Temp);
          Output << Temp;
		}
		else
		{
	      Output << Temp;
		}
      }
	  /*
	  for(int i = MOFAddr + 0xA; i<MOFSize + MOFAddr + 0xA; i = i+4)
      {
		Read_Mem_Dword(i, &temp1);
		Output << temp1;
	  }*/
    }
    Output.close();

  }
  return 0;
}


int main(int argc, CHAR* argv[])
{
  DWORD   ACPIAddr,RSDTAddr,FACPAddr,DSDTAddr;

  InitMem();
  ACPIAddr = GetACPITable();
  RSDTAddr = GetRSDTTable(ACPIAddr);
  FACPAddr = GetFACPTable(RSDTAddr);
  DSDTAddr = GetDSDTTable(FACPAddr);
  GetWMBC(DSDTAddr);
  GetWMIMOF(DSDTAddr);

  ofstream Output;
  Output.open("WmiFun.txt");
  if(Output.good())
  {
    for(int i=0; i<MethodCount; i++)
    {
	  Output <<"0x"<< hex << MethodID[i] <<" "<< MethodName[i] <<" 0x"<< hex << FunNum[i] << endl;
    }
  }
  Output.close();
exit:
  ReleaseMem();
  return 0;
}
 


